@@ -36,6 +36,10 @@
 #include "subprocs.h"
 #include "gs-debug.h"
 
+#ifdef HAVE_SHAPE_EXT
+#include <X11/extensions/shape.h>
+#endif
+
 static void gs_window_class_init (GSWindowClass *klass);
 static void gs_window_init       (GSWindow      *window);
 static void gs_window_finalize   (GObject       *object);
@@ -66,6 +70,7 @@ struct GSWindowPrivate
         guint64    logout_timeout;
         char      *logout_command;
         char      *keyboard_command;
+        char      *away_message;
 
         GtkWidget *vbox;
         GtkWidget *lock_box;
@@ -93,6 +98,10 @@ struct GSWindowPrivate
         gdouble    last_y;
 
         GTimer    *timer;
+
+#ifdef HAVE_SHAPE_EXT
+        int        shape_event_base;
+#endif
 };
 
 enum {
@@ -112,7 +121,8 @@ enum {
         PROP_KEYBOARD_COMMAND,
         PROP_LOGOUT_COMMAND,
         PROP_LOGOUT_TIMEOUT,
-        PROP_MONITOR
+        PROP_MONITOR,
+        PROP_AWAY_MESSAGE
 };
 
 static guint           signals [LAST_SIGNAL] = { 0, };
@@ -282,14 +292,41 @@ gs_window_clear (GSWindow *window)
         gdk_flush ();
 }
 
+static GdkRegion *
+get_outside_region (GSWindow *window)
+{
+        int i;
+        GdkRegion *region;
+
+        region = gdk_region_new ();
+        for (i = 0; i < window->priv->monitor; i++) {
+                GdkRectangle geometry;
+
+                gdk_screen_get_monitor_geometry (GTK_WINDOW (window)->screen,
+                                                   i, &geometry);
+                gdk_region_union_with_rect (region, &geometry);
+        }
+
+        return region;
+}
+
 static void
 update_geometry (GSWindow *window)
 {
         GdkRectangle geometry;
+        GdkRegion *outside_region, *monitor_region;
+
+        outside_region = get_outside_region (window);
 
         gdk_screen_get_monitor_geometry (GTK_WINDOW (window)->screen,
                                          window->priv->monitor,
                                          &geometry);
+        monitor_region = gdk_region_rectangle (&geometry);
+        gdk_region_subtract (monitor_region, outside_region);
+        gdk_region_destroy (outside_region); 
+
+        gdk_region_get_clipbox (monitor_region, &geometry);
+        gdk_region_destroy (monitor_region); 
 
         window->priv->geometry.x = geometry.x;
         window->priv->geometry.y = geometry.y;
@@ -610,6 +647,17 @@ x11_window_is_ours (Window window)
         return ret;
 }
 
+#ifdef HAVE_SHAPE_EXT
+static void
+unshape_window (GSWindow *window)
+{
+        gdk_window_shape_combine_region (GTK_WIDGET (window)->window,
+                                         NULL,
+                                         0,
+                                         0);
+}
+#endif
+
 static void
 gs_window_xevent (GSWindow  *window,
                   GdkXEvent *xevent)
@@ -646,6 +694,15 @@ gs_window_xevent (GSWindow  *window,
                         break;
                 }
         default:
+                /* extension events */
+#ifdef HAVE_SHAPE_EXT
+                if (ev->xany.type == (window->priv->shape_event_base + ShapeNotify)) {
+                        /*XShapeEvent *xse = (XShapeEvent *) ev;*/
+                        unshape_window (window);
+                        gs_debug ("Window was reshaped!");
+                }
+#endif
+
                 break;
         }
 
@@ -669,6 +726,7 @@ select_popup_events (void)
 
         gdk_error_trap_push ();
 
+        memset (&attr, 0, sizeof (attr));
         XGetWindowAttributes (GDK_DISPLAY (), GDK_ROOT_WINDOW (), &attr);
 
         events = SubstructureNotifyMask | attr.your_event_mask;
@@ -678,6 +736,25 @@ select_popup_events (void)
         gdk_error_trap_pop ();
 }
 
+static void
+window_select_shape_events (GSWindow *window)
+{
+#ifdef HAVE_SHAPE_EXT
+        unsigned long events;
+        int           shape_error_base;
+
+        gdk_error_trap_push ();
+
+        if (XShapeQueryExtension (GDK_DISPLAY (), &window->priv->shape_event_base, &shape_error_base)) {
+                events = ShapeNotifyMask;
+                XShapeSelectInput (GDK_DISPLAY (), GDK_WINDOW_XID (GTK_WIDGET (window)->window), events);
+        }
+
+        gdk_display_sync (gdk_display_get_default ());
+        gdk_error_trap_pop ();
+#endif
+}
+
 static void
 gs_window_real_show (GtkWidget *widget)
 {
@@ -701,6 +778,7 @@ gs_window_real_show (GtkWidget *widget)
         add_watchdog_timer (window, 30000);
 
         select_popup_events ();
+        window_select_shape_events (window);
         gdk_window_add_filter (NULL, (GdkFilterFunc)xevent_filter, window);
 }
 
@@ -744,72 +822,6 @@ gs_window_get_gdk_window (GSWindow *window)
         return GTK_WIDGET (window)->window;
 }
 
-static GPtrArray *
-get_env_vars (GtkWidget *widget)
-{
-        GPtrArray *env;
-        char      *str;
-        int        i;
-        static const char *allowed_env_vars [] = {
-                "PATH",
-                "LD_LIBRARY_PATH",
-                "SESSION_MANAGER",
-                "XAUTHORITY",
-                "XAUTHLOCALHOSTNAME",
-                "KRB5CCNAME",
-                "KRBTKFILE",
-                "LANG",
-                "LANGUAGE",
-                "LC_ALL",
-                "LC_CTYPE",
-                "LC_MESSAGES",
-                "LOGNAME",
-                "RUNNING_UNDER_GDM",
-                "GTK_DATA_PREFIX",
-                "GTK_EXE_PREFIX",
-                "GTK_IM_MODULE",
-                "GTK_IM_MODULE_FILE",
-                "GTK_MODULES",
-                "GTK_PATH",
-                "GTK2_RC_FILES",
-                "XDG_CONFIG_DIRS",
-                "XDG_DATA_DIRS",
-                "XDG_CACHE_HOME",
-                "XDG_CONFIG_HOME",
-                "XDG_DATA_HOME",
-                "MB_KBD_CONFIG",
-                "MB_KBD_VARIANT",
-                "MB_KBD_LANG",
-                "DBUS_SESSION_BUS_ADDRESS"
-        };
-
-        env = g_ptr_array_new ();
-
-        str = gdk_screen_make_display_name (gtk_widget_get_screen (widget));
-        g_ptr_array_add (env, g_strdup_printf ("DISPLAY=%s", str));
-        g_free (str);
-
-        g_ptr_array_add (env, g_strdup_printf ("HOME=%s",
-                                               g_get_home_dir ()));
-
-        for (i = 0; i < G_N_ELEMENTS (allowed_env_vars); i++) {
-                const char *var;
-                const char *val;
-                var = allowed_env_vars [i];
-                val = g_getenv (var);
-                if (val != NULL) {
-                        char *str;
-                        str = g_strdup_printf ("%s=%s", var, val);
-                        g_ptr_array_add (env, str);
-                        gs_debug ("adding environment: %s", str);
-                }
-        }
-
-        g_ptr_array_add (env, NULL);
-
-        return env;
-}
-
 /* just for debugging */
 static gboolean
 error_watch (GIOChannel   *source,
@@ -863,7 +875,6 @@ spawn_on_window (GSWindow *window,
 {
         int         argc;
         char      **argv;
-        GPtrArray  *env;
         GError     *error;
         gboolean    result;
         GIOChannel *channel;
@@ -880,13 +891,11 @@ spawn_on_window (GSWindow *window,
                 return FALSE;
         }
 
-        env = get_env_vars (GTK_WIDGET (window));
-
         error = NULL;
         result = gdk_spawn_on_screen_with_pipes (GTK_WINDOW (window)->screen,
                                                  NULL,
                                                  argv,
-                                                 (char **)env->pdata,
+                                                 NULL,
                                                  G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH,
                                                  NULL,
                                                  NULL,
@@ -896,11 +905,6 @@ spawn_on_window (GSWindow *window,
                                                  &standard_error,
                                                  &error);
 
-        for (i = 0; i < env->len; i++) {
-                g_free (g_ptr_array_index (env, i));
-        }
-        g_ptr_array_free (env, TRUE);
-
         if (! result) {
                 gs_debug ("Could not start command '%s': %s", command, error->message);
                 g_error_free (error);
@@ -1085,11 +1089,32 @@ wait_on_child (int pid)
         return status;
 }
 
+static void
+kill_keyboard_command (GSWindow *window)
+{
+        if (window->priv->keyboard_pid > 0) {
+                signal_pid (window->priv->keyboard_pid, SIGTERM);
+        }
+}
+
+static void
+kill_dialog_command (GSWindow *window)
+{
+        /* If a dialog is up we need to signal it
+           and wait on it */
+        if (window->priv->lock_pid > 0) {
+                signal_pid (window->priv->lock_pid, SIGTERM);
+        }
+}
+
 static void
 keyboard_command_finish (GSWindow *window)
 {
         g_return_if_fail (GS_IS_WINDOW (window));
 
+        /* send a signal just in case */
+        kill_keyboard_command (window);
+
         gs_debug ("Keyboard finished");
 
         if (window->priv->keyboard_pid > 0) {
@@ -1102,15 +1127,6 @@ keyboard_command_finish (GSWindow *window)
         }
 }
 
-static void
-kill_keyboard_command (GSWindow *window)
-{
-        if (window->priv->keyboard_pid > 0) {
-                signal_pid (window->priv->keyboard_pid, SIGTERM);
-        }
-        keyboard_command_finish (window);
-}
-
 static gboolean
 keyboard_command_watch (GIOChannel   *source,
                         GIOCondition  condition,
@@ -1224,7 +1240,12 @@ gs_window_dialog_finish (GSWindow *window)
         g_return_if_fail (GS_IS_WINDOW (window));
 
         gs_debug ("Dialog finished");
-        kill_keyboard_command (window);
+
+        /* make sure we finish the keyboard thing too */
+        keyboard_command_finish (window);
+
+        /* send a signal just in case */
+        kill_dialog_command (window);
 
         if (window->priv->lock_pid > 0) {
                 int exit_status;
@@ -1239,18 +1260,6 @@ gs_window_dialog_finish (GSWindow *window)
         remove_key_events (window);
 }
 
-static void
-kill_dialog_command (GSWindow *window)
-{
-        /* If a dialog is up we need to signal it
-           and wait on it */
-        if (window->priv->lock_pid > 0) {
-                signal_pid (window->priv->lock_pid, SIGTERM);
-        }
-
-        gs_window_dialog_finish (window);
-}
-
 /* very rudimentary animation for indicating an auth failure */
 static void
 shake_dialog (GSWindow *window)
@@ -1306,20 +1315,22 @@ lock_command_watch (GIOChannel   *source,
                 case G_IO_STATUS_NORMAL:
                         gs_debug ("command output: %s", line);
 
-                        if (strstr (line, "WINDOW ID=")) {
+                        if (strstr (line, "WINDOW ID=") != NULL) {
                                 guint32 id;
                                 char    c;
                                 if (1 == sscanf (line, " WINDOW ID= %" G_GUINT32_FORMAT " %c", &id, &c)) {
                                         create_lock_socket (window, id);
                                 }
-                        } else if (strstr (line, "NOTICE=")) {
-                                if (strstr (line, "NOTICE=AUTH FAILED")) {
+                        } else if (strstr (line, "NOTICE=") != NULL) {
+                                if (strstr (line, "NOTICE=AUTH FAILED") != NULL) {
                                         shake_dialog (window);
                                 }
-                        } else if (strstr (line, "RESPONSE=")) {
-                                if (strstr (line, "RESPONSE=OK")) {
+                        } else if (strstr (line, "RESPONSE=") != NULL) {
+                                if (strstr (line, "RESPONSE=OK") != NULL) {
+                                        gs_debug ("Got OK response");
                                         window->priv->dialog_response = DIALOG_RESPONSE_OK;
                                 } else {
+                                        gs_debug ("Got CANCEL response");
                                         window->priv->dialog_response = DIALOG_RESPONSE_CANCEL;
                                 }
                                 finished = TRUE;
@@ -1411,6 +1422,14 @@ popup_dialog_idle (GSWindow *window)
                 g_string_append_printf (command, " --logout-command='%s'", window->priv->logout_command);
         }
 
+	if (window->priv->away_message) {
+		char *quoted;
+
+		quoted = g_shell_quote (window->priv->away_message);
+		g_string_append_printf (command, " --away-message=%s", quoted);
+		g_free (quoted);
+	}
+
         if (is_user_switch_enabled (window)) {
                 command = g_string_append (command, " --enable-switch");
         }
@@ -1589,6 +1608,16 @@ gs_window_set_logout_command (GSWindow   *window,
         }
 }
 
+void
+gs_window_set_away_message (GSWindow   *window,
+                            const char *away_message)
+{
+        g_return_if_fail (GS_IS_WINDOW (window));
+
+        g_free (window->priv->away_message);
+        window->priv->away_message = g_strdup (away_message);
+}
+
 void
 gs_window_set_monitor (GSWindow *window,
                        int       monitor)
@@ -1640,6 +1669,9 @@ gs_window_set_property (GObject            *object,
         case PROP_LOGOUT_COMMAND:
                 gs_window_set_logout_command (self, g_value_get_string (value));
                 break;
+        case PROP_AWAY_MESSAGE:
+                gs_window_set_away_message (self, g_value_get_string (value));
+                break;
         case PROP_LOGOUT_TIMEOUT:
                 gs_window_set_logout_timeout (self, g_value_get_long (value));
                 break;
@@ -1678,6 +1710,9 @@ gs_window_get_property (GObject    *object,
         case PROP_LOGOUT_COMMAND:
                 g_value_set_string (value, self->priv->logout_command);
                 break;
+	case PROP_AWAY_MESSAGE: 
+		g_value_set_string (value, self->priv->away_message);
+		break;
         case PROP_LOGOUT_TIMEOUT:
                 g_value_set_long (value, self->priv->logout_timeout);
                 break;
@@ -2019,6 +2054,13 @@ gs_window_class_init (GSWindowClass *klass)
                                                               NULL,
                                                               NULL,
                                                               G_PARAM_READWRITE));
+        g_object_class_install_property (object_class,
+                                         PROP_AWAY_MESSAGE,
+                                         g_param_spec_string ("away-message",
+                                                              NULL,
+                                                              NULL,
+                                                              NULL,
+                                                              G_PARAM_READWRITE));
         g_object_class_install_property (object_class,
                                          PROP_KEYBOARD_ENABLED,
                                          g_param_spec_boolean ("keyboard-enabled",
@@ -2125,8 +2167,7 @@ gs_window_finalize (GObject *object)
 
         remove_command_watches (window);
 
-        kill_keyboard_command (window);
-        kill_dialog_command (window);
+        gs_window_dialog_finish (window);
 
         G_OBJECT_CLASS (gs_window_parent_class)->finalize (object);
 }
